from typing import Annotated
from fastapi import APIRouter, Depends, Security
from fastapi.responses import PlainTextResponse
from sqlmodel import Session, select

from cutt.db import Player, Team, engine
from cutt.security import change_password, get_current_active_user, read_player_me

P = Player

player_router = APIRouter(prefix="/player", tags=["player"])


def add_player(player: P):
    with Session(engine) as session:
        session.add(player)
        session.commit()


def add_player_to_team(player_id: int, team_id: int):
    with Session(engine) as session:
        player = session.exec(select(P).where(P.id == player_id)).one()
        team = session.exec(select(Team).where(Team.id == team_id)).one()
        if player and team:
            team.players.append(player)
            session.add(team)
            session.commit()
            return PlainTextResponse(
                f"added {player.display_name} ({player.username}) to {team.name}"
            )


def add_players(players: list[P]):
    with Session(engine) as session:
        for player in players:
            session.add(player)
        session.commit()


async def list_all_players():
    with Session(engine) as session:
        return session.exec(select(P)).all()


async def list_players(team_id: int):
    with Session(engine) as session:
        statement = select(Team).where(Team.id == team_id)
        players = [t.players for t in session.exec(statement)][0]
        if players:
            return [
                player.model_dump(include={"id", "display_name", "number"})
                for player in players
            ]


def read_teams_me(user: Annotated[P, Depends(get_current_active_user)]):
    with Session(engine) as session:
        return [p.teams for p in session.exec(select(P).where(P.id == user.id))][0]


player_router.add_api_route(
    "/add",
    endpoint=add_player,
    methods=["POST"],
    dependencies=[Security(get_current_active_user, scopes=["admin"])],
)
player_router.add_api_route(
    "/list/{team_id}",
    endpoint=list_players,
    methods=["GET"],
    dependencies=[Depends(get_current_active_user)],
)
player_router.add_api_route(
    "/list",
    endpoint=list_all_players,
    methods=["GET"],
    dependencies=[Security(get_current_active_user, scopes=["admin"])],
)
player_router.add_api_route(
    "/add/{player_id}/{team_id}",
    endpoint=add_player_to_team,
    methods=["GET"],
    dependencies=[Security(get_current_active_user, scopes=["admin"])],
)
player_router.add_api_route("/me", endpoint=read_player_me, methods=["GET"])
player_router.add_api_route("/me/teams", endpoint=read_teams_me, methods=["GET"])
player_router.add_api_route(
    "/change_password", endpoint=change_password, methods=["POST"]
)
